/*
 * CGI tictactoe - main routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "library.h"
#include "hash.h"
#include "rope.h"
#include "cgi.h"
#include "smalloc.h"
#include "log.h"

/* Must be bits and must agree with template.yy */
#define VAL_BLANK	(0)
#define VAL_USER	(1)
#define VAL_COMP	(2)

void template_parse(hash_t *cgihash, char *square, int vu, int vc);

static void
square_write(char *square, char *istr, int pval)
{
	int	 val;
	int	 i;

	val = atol(istr);

	for (i = 0; i < 9; i++) {
		if (val&1) square[i] = pval;
		val >>= 1;
	}
}

static int
square_win(char *square)
{
	int	 val;
	int	 i;
	int	 j;

	for (i = 0; i < 3; i++) {
		/* Check	0,1,2	3,4,5	6,7,8 */
		val = 255;	/* Lots of bits set */
		for (j = 0; j < 3 ; j++) val &= square[(3 * i) + j];
		if (val) return val;

		/* Check	0,3,6	1,4,7	2,5,8 */
		val = 255;	/* Lots of bits set */
		for (j = 0; j < 3 ; j++) val &= square[i + (3 * j)];
		if (val) return val;
	}

	/* Check 0,4,8 */
	val = 255;
	for (j = 0; j < 3; j++) val &= square[(4 * j)];
	if (val) return val;

	/* Check 2,4,6 */
	val = 255;
	for (j = 0; j < 3; j++) val &= square[(2 * j) + 2];
	if (val) return val;

	return 0;
}

int
square_state(char *square)
{
	int	 val;
	int	 i;

	val = square_win(square);
	if (val) return val;
	
	for (i = 0; i < 9; i++)
		if (square[i] == VAL_BLANK)
			return VAL_BLANK;
	
	return -1;	/* Game is a draw */
}

static int
square_find(char *square, int user)
{
	int		 i;

	for (i = 0; i < 9; i++) {
		if (square[i] == VAL_BLANK) {
			square[i] = user;
			/* Not the most efficient, but it works */
			if (square_win(square) ==  user)
				return i;
			square[i] = VAL_BLANK;
		}
	}

	return -1;
}

static int
square_liked(char *square)
{
	return square_find(square, VAL_COMP);
}

static int
square_forced(char *square)
{
	return square_find(square, VAL_USER);
}

/* We assume at least one empty square */
static bool
square_cmove(char *square, hash_t *cgihash)
{
	unsigned char	 order[] = { 4, 0, 2, 6, 8, 1, 3, 5, 7 };
	int				 off;
	int				 i;

	if ((off = square_forced(square)) != -1) {
		square[off] = VAL_COMP;
	}
	else if ((off = square_liked(square)) != -1) {
		square[off] = VAL_COMP;
	}
	else {
		for (i = 0; i < 9; i++) {
			if (square[order[i]] == VAL_BLANK) {
				square[order[i]] = VAL_COMP;
				break;
			}
		}
	}

	return TRUE;
}

static void
temp_square_print(char *square)
{
	int	 i;
	int	 j;

	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++)
			fprintf(stderr, "%d ", square[(3 * i) + j]);
		fprintf(stderr, "\n");
	}
}

static void
square_parse(hash_t *cgihash, char *square)
{
	int		 vu = 0;
	int		 vc = 0;
	int		 i;

	for (i = 9; i > -1; i--) {
		vu <<= 1;
		vc <<= 1;
		switch(square[i]) {
			case VAL_USER:
				vu |= 1;
				break;
			case VAL_COMP:
				vc |= 1;
				break;
		}
	}

	template_parse(cgihash, square, vu, vc);
}

void
invoke(void)
{
	hash_t	*cgihash;
	char	 square[9];
	char	*temp;

	cgihash = cgi_hash_new();

	memset(square, VAL_BLANK, sizeof(square));

	/* Player is 'x', we are 'o' */
	temp = hash_get(cgihash, "u");
	if (temp)
		square_write(square, temp, VAL_USER);

	temp = hash_get(cgihash, "c");
	if (temp)
		square_write(square, temp, VAL_COMP);

	if (square_state(square) == VAL_BLANK)
		square_cmove(square, cgihash);

	// temp_square_print(square);

	square_parse(cgihash, square);
}
