/*
 * Virtual Card CGI - main routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "library.h"
#include "log.h"
#include "hash.h"
#include "cgi.h"

#define EMAILMSG "'%s' does not appear to be a valid email address, " \
	"please press the back button in your browser to amend the form."

hash_t	*file_read(hash_t *cgihash);
void	 file_write(hash_t *cgihash, hash_t *card);
bool	 template_parse(hash_t *cgihash, hash_t *card);
bool	 email_send(hash_t *cgihash);

void
user_message(hash_t *cgihash, char *errmsg)
{
	hash_add(cgihash, "_template", sstrdup("message"));
	hash_add(cgihash, "Message", sstrdup(errmsg));

	if (!template_parse(cgihash, cgihash)) {
		printf("Content-type: text/html\n\n");
		printf("<HTML>\n");
		printf("<HEAD><TITLE>Message</TITLE></HEAD>\n");
		printf("<BODY>Error: could not parse template.<BR>\n");
		printf("Original message was:<BR>\n");
		printf("%s\n", errmsg);
		printf("</BODY>\n</HTML>\n\n");
	}
}

void
main_build(hash_t *cgihash)
{
	bool	 b = FALSE;
	char	*t_email;
	char	*temp;
	char	 buf[BUFSIZ];

	/* We essentially use the CGI hash as the card, it does not
	 * matter if we put extra keys into the file, they are ignored.
	 * _template is not stored, as a special case, because it
	 * has other consequences. 'action' is always 'build' */

	file_write(cgihash, cgihash);

	t_email = hash_get(cgihash, "t_email");
	if (!t_email) {
		log("cgiconf", PRI_CRIT, "No t_email field in hash table. "
						"Host %s, case %s, template %s",
						(char *)hash_get(cgihash, ENV_HOSTNAME),
						(char *)hash_get(cgihash, "case"),
						(char *)hash_get(cgihash, "template"));
	}
	else if (! (temp = strchr(t_email, '@')) ) {
		snprintf(buf, BUFSIZ - 1, EMAILMSG, t_email);
		buf[BUFSIZ - 1] = 0;
		user_message(cgihash, buf);
	}
	else if (!strchr(temp, '.')) {
		snprintf(buf, BUFSIZ - 1, EMAILMSG, t_email);
		buf[BUFSIZ - 1] = 0;
		user_message(cgihash, buf);
	}
	else if ( (b = email_send(cgihash)) ) {
		snprintf(buf, BUFSIZ - 1, "A card was created at "
					"http://%s/cgi-bin/vcard.cgi?case=%s&card=%s "
					"for later collection. An email notification "
					"has been sent to %s",
					(char *)hash_get(cgihash, ENV_HOSTNAME),
					(char *)hash_get(cgihash, "case"),
					(char *)hash_get(cgihash, "card"),
					t_email);
		user_message(cgihash, buf);
	}
	else {
		snprintf(buf, BUFSIZ - 1, "A card was created at "
					"http://%s/cgi-bin/vcard.cgi?case=%s&card=%s "
					"for later collection. <B>An email notification "
					"could not be sent to %s</B>",
					(char *)hash_get(cgihash, ENV_HOSTNAME),
					(char *)hash_get(cgihash, "case"),
					(char *)hash_get(cgihash, "card"),
					t_email);
		user_message(cgihash, buf);
	}
}

void
main_test(hash_t *cgihash)
{
	template_parse(cgihash, cgihash);
}

void
main_read(hash_t *cgihash)
{
	char	*file;
	hash_t	*card;

	file = hash_get(cgihash, "card");
	if (!file) {
		user_message(cgihash, "No 'card' parameter in hash table");
		return;
	}

	// DPRINTF((stderr, "Got card name %s\n", file));

	card = file_read(cgihash);
	if (!card) {
		user_message(cgihash, "No card was found.");
		return;
	}

	template_parse(cgihash, card);
}

void
invoke(void)
{
	hash_t	*cgihash;
	char	*hostname;
	char	*instance;
	char	*action;

	cgihash = cgi_hash_new();

	instance = hash_get(cgihash, "case");
	if (!instance) {
		log("cgiconf", PRI_CRIT, "No 'case' parameter in hash table");
		exit(0);
	}

	hostname = getenv(ENV_HOSTNAME);
	if (!hostname) {
		log("cgisys", PRI_CRIT, "Could not get " ENV_HOSTNAME);
		exit(0);
	}
	hash_add(cgihash, ENV_HOSTNAME, sstrdup(hostname));

	action = hash_get(cgihash, "action");
	if (!action)
		main_read(cgihash);
	else if (strcmp(action, "read") == 0)
		main_read(cgihash);
	else if (strcmp(action, "build") == 0)
		main_build(cgihash);
	else if (strcmp(action, "test") == 0)
		main_test(cgihash);
	else {
		log("cgisys", PRI_WARNING, "Unrecognised 'action' : %s",
						action);
		exit(0);
	}
}
