%{
/*
 * Virtual Card CGI - template parser
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "library.h"
#include "hash.h"
#include "rope.h"
#include "cgi.h"
#include "log.h"

static void template_field(char *str);

hash_t		*tpt_cgihash;
hash_t		*tpt_card;

%}

%option nounput
%option noyywrap

tVALIDCHAR	[a-zA-Z0-9_]
tTOKEN		\{{tVALIDCHAR}+\}

%%

{tTOKEN}		{ template_field(yytext);	}

%%

static void
template_field(char *str)
{
	char	*data;

	str++;
	str[strcspn(str, "}")] = 0;

	data = hash_get(tpt_card, str);
	if (data) {
		fprintf(yyout, "%s", data);
	}
	else {
		log("cgiconf", PRI_WARNING, "No '%s' field in card", str);
		/* XXX MORE diagnostics */
		fprintf(yyout, "No '%s' field in card", str);
	}
}

static FILE *
template_stream(hash_t *cgihash, hash_t *card)
{
	rope_t	*rope;
	char	*hostname;
	char	*instance;
	char	*template;
	FILE	*input;

	hostname = hash_get(cgihash, ENV_HOSTNAME);
	instance = hash_get(cgihash, "case");

	template = hash_get(card, "_template");
	if (!template)
		template = hash_get(card, "template");
	if (!template)
		template = "default";

	rope = rope_new(DIR_DATA "/");
	rope_add(rope, hostname);
	rope_add(rope, "/vcard/");
	rope_add(rope, instance);
	rope_add(rope, "/templates/");
	rope_add(rope, template);
	rope_add(rope, ".tpt");

	// DPRINTF((stderr, "Parsing from %s\n", rope_chars(rope)));

	input = fopen(rope_chars(rope), "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open file %s : %s",
						rope_chars(rope), strerror(errno));
		return NULL;
	}

	rope_delete(rope);

	return input;
}

bool
template_parse(hash_t *cgihash, hash_t *card)
{
	yyin = template_stream(cgihash, card);
	if (!yyin)
		return FALSE;

	tpt_cgihash = cgihash;
	tpt_card = card;

	printf("Content-type: text/html\n\n");
	yylex();

	return TRUE;
}
