#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "library.h"
#include "vector.h"
#include "log.h"
#include "smalloc.h"

/* First part */
#define CCODEOFF	(19)
#define CCODELEN	(2)

/* Second part */
#define CNUMLEN	(3)
#define DATELEN (10)
#define DATEOFF	(3)
#define DAYOFF	(19)
#define NIGHTOFF	(27)

const char *wcodes[] = {
   [65] = "Fair",
   [66] = "Partly Cloudy",
   [67] = "Cloudy",
   [68] = "Dust",
   [69] = "Mostly Cloudy",
   [70] = "Foggy",
   [71] = "Hot",
   [72] = "Hazy",
   [73] = "Very Cold",
   [74] = "Snow Shower",
   [75] = "Smoke",
   [76] = "Drizzle",
   [77] = "Flurries",
   [78] = "Windy",
   [79] = "Mixed",
   [80] = "Blizzard",
   [81] = "Blowing Snow",
   [82] = "Rain",
   [83] = "Snow",
   [84] = "Thunderstorm",
   [85] = "Sunny",
   [87] = "Showers",
   [88] = "Sleet",
   [89] = "Freezing Rain",
   [90] = "Freezing Drizzle",
};

void
input_process(vector_t *dv, char *line)
{
	int			 val;
	const char	*daytext;
	const char	*nighttext;

	val = atol(&line[DAYOFF]);
	daytext = wcodes[val];

	val = atol(&line[NIGHTOFF]);
	nighttext = wcodes[val];

	printf("Day: %s\nNight: %s\n", daytext, nighttext);
}

bool
input_parse(FILE *input)
{
	vector_t	*dv;
	char		 line[BUFSIZ];
	int			 numcities;
	char		 ccode[CCODELEN + 1];

	if (!fgets(line, BUFSIZ, input)) {
		log("cgisys", PRI_CRIT, "Unexpected EOF at start of file");
		return FALSE;
	}

	numcities = atol(line);

	dv = vector_new(sfree, numcities);

	while (numcities--) {
		if (!fgets(line, BUFSIZ, input)) {
			log("cgisys", PRI_CRIT, "Unexpected EOF in cities");
			return FALSE;
		}
		if (strlen(line) < 22) {
			log("cgisys", PRI_CRIT, "Unexpected short line '%s'", line);
			return FALSE;
		}
		strncpy(ccode, &line[CCODEOFF], CCODELEN);
		ccode[CCODELEN] = 0;
		vector_push_back(dv, sstrdup(ccode));
	}

	while (strlen(line)) {
		if (!fgets(line, BUFSIZ, input)) {
			log("cgisys", PRI_CRIT, "Unexpected EOF before data");
			return FALSE;
		}
	}

	while (fgets(line, BUFSIZ, input)) {
		input_process(dv, line);
	}

	return TRUE;
}
